//
//  MCHelperTool.m
//  MemoryCell
//
//  Created by Quentin Carnicelli on 9/8/06.
//  Copyright 2006 Rogue Amoeba Software, LLC. All rights reserved.
//

#import "MCHelperTool.h"
#import "MCMemoryStats.h"

#define _kIdleTimerInterval 60.f

@implementation MCHelperTool

- (id)init
{
	if( (self = [super init]) != nil )
	{
		_lastRequestTimestamp = [NSDate timeIntervalSinceReferenceDate];
	}
	
	return self;
}

- (NSData*)processMemoryStatsForPid: (pid_t)pid lastStats: (NSData*)lastStats
{
	kern_return_t err;
	process_memory_stats oldStats;
	process_memory_stats newStats;
	NSData* data;
	
	_lastRequestTimestamp = [NSDate timeIntervalSinceReferenceDate];
	
	if( lastStats )
	{
		err = process_mem_stats_from_data( (CFDataRef)lastStats, &oldStats );
		if( err )
			lastStats = nil;
	}
		
	if( lastStats )
		err = process_get_mem_stats( pid, &oldStats, &newStats );
	else
		err = process_get_mem_stats( pid, nil, &newStats );

	if( err )
		return nil;

	err = process_mem_stats_as_data( &newStats, (CFDataRef*)&data );
	if( err )
		return nil;
	
	return [data autorelease];
}

- (void)_idle: (id)sender
{
	NSTimeInterval timeSinceLastRequest = [NSDate timeIntervalSinceReferenceDate] - _lastRequestTimestamp;
	
	if( timeSinceLastRequest > _kIdleTimerInterval )
	{
		NSLog( @"No requests for %.0f seconds, bailing", timeSinceLastRequest );
		[self terminate];
	}
}

- (void)terminate
{
	[[NSApplication sharedApplication] performSelector: @selector(terminate:) withObject: nil afterDelay: 0];
}

@end

static int _runServer(void)
{
	MCHelperTool* helperTool = [[MCHelperTool alloc] init];

	NSConnection* connection = [[NSConnection alloc] init];

	[connection setRootObject: [NSProtocolChecker
									protocolCheckerWithTarget: helperTool
									protocol: @protocol( MCHelperToolInterface )]];

	if( ![connection registerName: [NSString stringWithFormat: MCHelperPortNameFormat, getpid()]] )
		return -1;

	[NSTimer scheduledTimerWithTimeInterval: _kIdleTimerInterval target: helperTool selector: @selector(_idle:) userInfo: NO repeats: YES];

	[[NSApplication sharedApplication] run];

	[connection release];

	[helperTool release];
	
	return -1;
}

static int _runSelfRepair(void)
{
	NSError* error = nil;
	NSString* path = [[NSBundle mainBundle] executablePath];

	NSArray *commands = [NSArray arrayWithObjects:
		[NSArray arrayWithObjects: @"/bin/chmod", [NSString stringWithFormat: @"%o", MCHelperToolFilePermissions], path, nil],
		[NSArray arrayWithObjects: @"/usr/bin/chgrp", @MCHelperToolFileGroupOwner, path, nil],
		nil];

	if( getgrnam( MCHelperToolFileGroupOwner ) == nil )
		return -1;

	if( !MCRunAuthorizedCommandSequence( commands, @"MemoryCell must set custom file permissions (setgid/procview) to operate correctly.\n\n", &error ) )
	{
		NSLog( @"%@", error );
		return -1;
	}
	
	return 0;
}

int main(int argc, char *argv[])
{
	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
	int i, err;
	BOOL doSelfRepair;
	
	doSelfRepair = NO;
	for( i = 1; i < argc; i++ )
	{
		if( strcasecmp( argv[i], "--self-repair" ) == 0 )
		{
			doSelfRepair = YES;
			break;
		}
	}

	if( doSelfRepair )
	{
		err = _runSelfRepair();
	}
	else
	{
		err = _runServer();
	}

	[pool release];

	return err;
}
